# Generated by Django 4.0.10 on 2023-06-06 16:44

from django.db import migrations, models
import djangocms_text_ckeditor.fields
import smart_media.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0009_alter_socialsharing_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='AppsBanner',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('application_type', models.CharField(choices=[('free_apply', 'Free apply'), ('job_offers', 'Job offers'), ('news', 'News'), ('trombinoscope', 'Trombinoscope')], max_length=13, unique=True)),
                ('image', smart_media.modelfields.SmartMediaField(blank=True, null=True, upload_to='firm/apps_banner/%y/%m', verbose_name='Image')),
                ('title', models.CharField(blank=True, max_length=150, null=True, verbose_name='Title')),
                ('description', djangocms_text_ckeditor.fields.HTMLField(blank=True, null=True, verbose_name='Description')),
            ],
            options={
                'verbose_name': 'App banner',
                'verbose_name_plural': 'App banners',
            },
        ),
    ]



from django.conf import settings
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class FirmInfoAppConfig(AppConfig):
    name = "firm_info"
    verbose_name = _(settings.CLIENT_NAME)
    verbose_name_plural = _(settings.CLIENT_NAME)



# Generated by Django 4.0.10 on 2023-05-26 11:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0004_firmcontact_logo_firmcontact_logo_invert'),
    ]

    operations = [
        migrations.AddField(
            model_name='firmcontact',
            name='favicon',
            field=models.ImageField(blank=True, null=True, upload_to='firm/logo_inverted'),
        ),
    ]



"""
Default application settings
----------------------------

These are the default settings you can override in your own project settings
right after the line which load the default app settings.

"""
BLOG_PAGINATION = 5
"""
Blog entry per page limit for pagination, set it to ``None`` to disable
pagination.
"""

ARTICLE_PAGINATION = 6
"""
Article entry per page limit for pagination, set it to ``None`` to disable
pagination.
"""



# Generated by Django 4.0.10 on 2023-06-12 13:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0011_alter_firmcontact_favicon_alter_firmcontact_logo_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='appsbanner',
            name='application_type',
            field=models.CharField(choices=[('application_sent', 'Application sent'), ('free_apply', 'Free apply'), ('job_offers', 'Job offers'), ('news', 'News'), ('trombinoscope', 'Trombinoscope')], max_length=16, unique=True),
        ),
    ]



# Generated by Django 4.0.10 on 2023-06-02 13:09

from django.db import migrations, models
import smart_media.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0006_alter_firmcontact_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='SocialSharing',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('og_image', smart_media.modelfields.SmartMediaField(blank=True, null=True, upload_to='firm/social_sharing/%y/%m', verbose_name='OG Image')),
                ('og_description', models.TextField(blank=True, default='', max_length=180, verbose_name='OG Description')),
                ('og_twitter_site', models.CharField(blank=True, default='', max_length=100, verbose_name='OG Twitter Site')),
            ],
            options={
                'verbose_name': 'Social sharing',
                'verbose_name_plural': 'Social shares',
            },
        ),
        migrations.CreateModel(
            name='Tracking',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('tag_analytic', models.CharField(blank=True, default='', max_length=100, verbose_name='Tag Analytic')),
            ],
            options={
                'verbose_name': 'Tracking',
                'verbose_name_plural': 'Trackings',
            },
        ),
    ]



# Generated by Django 4.0.10 on 2023-05-22 15:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0002_alter_firmcontact_options_alter_firmcontact_address_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='firmcontact',
            name='baseline',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AddField(
            model_name='firmcontact',
            name='short_description',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='link',
            name='name',
            field=models.CharField(choices=[('linkedin', 'linkedin'), ('facebook', 'facebook'), ('twitter', 'twitter'), ('youtube', 'youtube'), ('instagram', 'instagram'), ('glassdoor', 'glassdoor'), ('bitbucket', 'bitbucket'), ('github', 'github'), ('gitlab', 'gitlab'), ('tiktok', 'tiktok'), ('twitch', 'twitch'), ('discord', 'discord'), ('vk', 'vk'), ('slack', 'slack'), ('whatsapp', 'whatsapp'), ('weechat', 'weechat')], max_length=255),
        ),
    ]



# Generated by Django 4.0.10 on 2023-06-02 15:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0008_alter_tracking_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='socialsharing',
            options={'verbose_name': 'Social media share', 'verbose_name_plural': 'Social media shares'},
        ),
    ]



import contextlib
from django.core.exceptions import ObjectDoesNotExist
from django.template import Library, loader
from firm_info.models import AppsBanner, FirmContact, Link, SocialSharing, Tracking
from firm_info.serializers import (
    serialize_firm_apps_banner,
    serialize_firm_description,
    serialize_firm_info,
    serialize_firm_social,
    serialize_firm_social_sharing,
)

register = Library()


@register.simple_tag(takes_context=True, name="firm_contact")
def firm_contact(context, template_path):
    """
    Renders the template which path is provided as param
    using FirmContact only instance serialized contact data.

    Args:
        template_path (str): The path to the template file.

    Returns:
        str: The rendered HTML output of the firm contact informations.

    Usage:

    .. code-block:: html

        {% load firm_info %}
        {% firm_contact "path/to/template.html" %}

    """
    qs_firm_info = FirmContact.objects.all()
    if qs_firm_info.exists():
        template = loader.get_template(template_path)
        specific_context = serialize_firm_info(qs_firm_info)
        combined_context = {**context.flatten(), **specific_context}
        rendered = template.render(combined_context)
        return rendered
    else:
        return ''


@register.simple_tag(takes_context=True, name="firm_social_links")
def firm_social_links(context, template_path):
    """
    Renders the template which path is provided as param
    using all social network link objects serialized data
    related the only FirmContact instance.

    Args:
        template_path (str): The path to the template file.

    Returns:
        str: The rendered HTML output of the firm social networks links.

    Usage:

    .. code-block:: html

        {% load firm_info %}
        {% firm_social_links "path/to/template.html" %}

    """
    links = Link.objects.all()
    if links.exists():
        template = loader.get_template(template_path)
        specific_context = serialize_firm_social(links)
        combined_context = {**context.flatten(), **specific_context}
        rendered = template.render(combined_context)
        return rendered
    else:
        return ''


@register.simple_tag(takes_context=True, name="firm_description")
def firm_description(context, template_path):
    """
    Renders the template which path is provided as param
    using FirmContact only instance serialized description data.

    Args:
        template_path (str): The path to the template file.

    Returns:
        str: The rendered HTML output of the firm description.

    Usage:

    .. code-block:: html

        {% load firm_info %}
        {% firm_description "path/to/template.html" %}

    """
    qs_firm_info = FirmContact.objects.all()
    if qs_firm_info.exists():
        template = loader.get_template(template_path)
        specific_context = serialize_firm_description(qs_firm_info)
        combined_context = {**context.flatten(), **specific_context}
        rendered = template.render(combined_context)
        return rendered
    else:
        return ''


@register.simple_tag(takes_context=True, name="firm_logos")
def firm_logos(context, template_path):
    """
    Renders the firm logos using the specified template.

    Args:
        template_path (str): The path to the template file.

    Returns:
        str: The rendered HTML output of the firm logos.

    Usage:

    .. code-block:: html

        {% load firm_info %}
        {% firm_logos "path/to/template.html" %}

    """
    firm_instance = FirmContact.objects.first()
    if firm_instance:
        template = loader.get_template(template_path)
        specific_context = {
            "logo": getattr(firm_instance, "logo", None),
            "logo_invert": getattr(firm_instance, "logo_invert", None),
            "favicon": getattr(firm_instance, "favicon", None),
        }
        combined_context = {**context.flatten(), **specific_context}
        rendered = template.render(combined_context)
        return rendered
    else:
        return ''


@register.simple_tag(takes_context=True, name="firm_social_shares")
def firm_social_shares(context, template_path):
    """
    Renders the template which path is provided as param
    using all social network shares link objects serialized data
    related the only SocialSharing instance.

    Args:
        template_path (str): The path to the template file.

    Returns:
        str: The rendered HTML output of the firm social media shares.

    Usage:

    .. code-block:: html

        {% load firm_info %}
        {% firm_social_shares "path/to/template.html" %}

    """
    social_shares = SocialSharing.objects.first()

    if social_shares:
        template = loader.get_template(template_path)
        specific_context = serialize_firm_social_sharing(social_shares)
        combined_context = {**context.flatten(), **specific_context}
        rendered = template.render(combined_context)

        return rendered
    else:
        return ''


@register.filter("firm_tag_analytic")
def firm_tag_analytic(value=None):
    """
    Filters the firm tag analytic value.

    Args:
        value: The input value (not used).

    Returns:
        str: The tag analytic value from the first Tracking object if it exists,
        otherwise an empty string.

    Usage:

    .. code-block:: html

        {% load firm_info %}
        {% firm_tag_analytic "path/to/template.html" %}

    """
    return Tracking.objects.first().tag_analytic if Tracking.objects.exists() else ""


@register.simple_tag(takes_context=True, name="app_banner")
def app_banner(context, app_type, template_path):
    """
    Renders the app banner using the specified template and application type.

    Args:
        app_type (str): The application type.
        template_path (str): The path to the template file.

    Returns:
        str: The rendered HTML output of the app banner.

    Usage:

    .. code-block:: html

        {% load firm_info %}
        {% app_banner "path/to/template.html" %}

    """
    context = {}
    template = loader.get_template(template_path)

    with contextlib.suppress(ObjectDoesNotExist):
        app_banner = AppsBanner.objects.get(application_type=app_type)
        specific_context = serialize_firm_apps_banner(app_banner)
        combined_context = {**context.flatten(), **specific_context}
    rendered = template.render(combined_context)

    return rendered



from firm_info.exceptions import SerializeFirmError


def _format_address(firm_info: dict) -> str:
    """
    Formats the address using the firm information.

    Args:
        firm_info (dict): The firm information with the following keys:

            - "address": The address of the firm.
            - "postal_code": The postal code of the firm.
            - "city": The city of the firm.
            - "country": The country of the firm.

    Returns:
        str: The formatted address string in the format
          `"{address}, {postal_code} {city} {country}"`.
    """
    return "{}, {} {} {}".format(
        firm_info.get("address"),
        firm_info.get("postal_code"),
        firm_info.get("city"),
        firm_info.get("country"),
    )


def serialize_firm_info(queryset):
    """
    Serializes the firm information from a queryset.

    Args:
        queryset: The queryset containing the firm information.

    Returns:
        dict: the serialized firm information with the following keys:

            - "email": The email address of the firm.
            - "phone": The phone number of the firm.
            - "full_address": The formatted full address of the firm.
            - "address": The address of the firm.
            - "postal_code": The postal code of the firm.
            - "city": The city of the firm.
            - "country": The country of the firm.

    Raises:
        SerializeFirmError: Raised when an error occurs during serialization.
    """

    try:
        firm_info = queryset.values(
            "phone_number", "email", "address", "postal_code", "city", "country"
        ).first()
        return {
            "email": firm_info.get("email"),
            "phone": firm_info.get("phone_number"),
            "full_address": _format_address(firm_info),
            "address": firm_info.get("address"),
            "postal_code": firm_info.get("postal_code"),
            "city": firm_info.get("city"),
            "country": firm_info.get("country"),
        }
    except Exception as err:
        raise SerializeFirmError from err


def serialize_firm_social(queryset):
    """
    Serializes the firm social media information from a queryset.

    Args:
        queryset: The queryset containing the firm social media information.

    Returns:
        dict: The serialized firm social media information with the social media names
        as keys and their corresponding URLs as values.

    Raises:
        SerializeFirmError: Raised when an error occurs during serialization.
    """

    try:
        firm_socials = list(
            queryset.values(
                "name",
                "url",
            )
        )
        return {
            social.get("name", "NOTFOUND"): social.get("url", "NOTFOUND")
            for social in firm_socials
        }
    except Exception as err:
        raise SerializeFirmError from err


def serialize_firm_description(queryset):
    """
    Serializes the firm description from a queryset.

    Args:
        queryset: The queryset containing the firm description.

    Returns:
        dict: The serialized firm description with the following keys:

            - "baseline": The baseline of the firm.
            - "short_description": The short description of the firm.

    Raises:
        SerializeFirmError: Raised when an error occurs during serialization.
    """
    try:
        firm_info = queryset.values("baseline", "short_description").first()
        return {
            "baseline": firm_info.get("baseline"),
            "short_description": firm_info.get("short_description"),
        }
    except Exception as err:
        raise SerializeFirmError from err


def serialize_firm_social_sharing(obj):
    """
    Serialize Firm social networks sharing URLs.

    Args:
        obj (QuerySet): The `SocialSharing` queryset.

    Returns:
        dict: The serialized data with the following keys:

            - og_image (SmartMediaField): The OG image for social media sharing.
            - og_description (TextField): The OG description for social media sharing.
            - og_twitter_site (CharField): The OG Twitter site for social media sharing.

    Raises:
        SerializeFirmError: Raised when an error occurs during serialization.
    """

    try:
        return {
            "og_image": obj.og_image,
            "og_description": obj.og_description,
            "og_twitter_site": obj.og_twitter_site,
        }

    except Exception as err:
        raise SerializeFirmError from err


def serialize_firm_apps_banner(obj):
    """
    Serializes an instance of the AppsBanner model into a dictionary.

    Args:
        obj: The AppsBanner object to be serialized.

    Returns:
        dict: The serialized data with the following keys:

            - "title": The title of the `AppsBanner`.
            - "description": The description of the AppsBanner.
            - "image": The image associated with the AppsBanner.

    Raises:
        SerializeFirmError: Raised when an error occurs during serialization.
"""

    try:
        return {
            "title": obj.title,
            "description": obj.description,
            "image": obj.image,
        }

    except Exception as err:
        raise SerializeFirmError from err



# Generated by Django 4.0.10 on 2023-05-15 15:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FirmContact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('phone_number', models.CharField(max_length=20)),
                ('email', models.EmailField(max_length=254)),
                ('address', models.CharField(max_length=255)),
                ('postal_code', models.CharField(max_length=20)),
                ('city', models.CharField(max_length=100)),
                ('country', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'Compagny contact informations',
            },
        ),
        migrations.CreateModel(
            name='Link',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(choices=[('linkedin', 'linkedin'), ('facebook', 'facebook'), ('twitter', 'twitter'), ('youtube', 'youtube'), ('instagram', 'instagram')], max_length=255)),
                ('url', models.URLField(max_length=255)),
                ('client_contact', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='firm_info.firmcontact')),
            ],
            options={
                'verbose_name': 'Social network link',
                'verbose_name_plural': 'Social networks links',
            },
        ),
    ]



# Generated by Django 4.0.10 on 2023-05-17 11:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='firmcontact',
            options={'verbose_name': 'Compagny contact information', 'verbose_name_plural': 'Compagny contact information'},
        ),
        migrations.AlterField(
            model_name='firmcontact',
            name='address',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='firmcontact',
            name='city',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='firmcontact',
            name='country',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='firmcontact',
            name='email',
            field=models.EmailField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='firmcontact',
            name='phone_number',
            field=models.CharField(blank=True, default='', max_length=20),
        ),
        migrations.AlterField(
            model_name='firmcontact',
            name='postal_code',
            field=models.CharField(blank=True, default='', max_length=20),
        ),
    ]



from ..defaults import (
    CLIENT_NAME,
)


class FirmInfosDefaultSettings:
    """
    Default application settings class to use with a "django-configuration" class.

    Example:

        You just have to inherit from it in your settings class: ::

            from configurations import Configuration
            from firm_info.contrib.django_configuration import FirmInfosDefaultSettings

            class Dev(FirmInfosDefaultSettings, Configuration):
                CLIENT_NAME = "Myclient"

        This will override the setting ``CLIENT_NAME``
    """  # noqa: E501

    CLIENT_NAME = CLIENT_NAME



from django.db import models
from django.db.models.signals import post_delete, pre_save
from django.utils.translation import gettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField
from firm_info.managers import SingletonManager
from smart_media.modelfields import SmartMediaField
from smart_media.signals import auto_purge_files_on_change, auto_purge_files_on_delete


class FirmContact(models.Model):
    """
    Represents the contact information for a firm.

    Args:
        models.Model: The base model class provided by Django.

    Attributes:
        phone_number (CharField): The phone number of the firm.
        email (EmailField): The email address of the firm.
        address (CharField): The address of the firm.
        postal_code (CharField): The postal code of the firm.
        city (CharField): The city of the firm.
        country (CharField): The country of the firm.
        baseline (CharField): The baseline of the firm.
        short_description (TextField): The short description of the firm.
        logo (SmartMediaField): The logo of the firm.
        logo_invert (SmartMediaField): The inverted logo of the firm.
        favicon (SmartMediaField): The favicon of the firm.
        objects (SingletonManager): The manager for the FirmContact model.
    """
    phone_number = models.CharField(max_length=20, null=False, blank=True, default="")
    email = models.EmailField(null=False, blank=True, default="")
    address = models.CharField(max_length=255, null=False, blank=True, default="")
    postal_code = models.CharField(max_length=20, null=False, blank=True, default="")
    city = models.CharField(max_length=100, null=False, blank=True, default="")
    country = models.CharField(max_length=100, null=False, blank=True, default="")
    baseline = models.CharField(max_length=255, null=False, blank=True, default="")
    short_description = models.TextField(null=False, blank=True, default="")
    logo = SmartMediaField(
        _("Logo"),
        null=True,
        blank=True,
        upload_to="firm_info/firm_contact/logo/%y/%m",
    )
    logo_invert = SmartMediaField(
        _("Logo invert"),
        null=True,
        blank=True,
        upload_to="firm_info/firm_contact/logo_invert/%y/%m",
    )
    favicon = SmartMediaField(
        _("Favicon"),
        null=True,
        blank=True,
        upload_to="firm_info/firm_contact/favicon/%y/%m",
    )

    objects = SingletonManager()

    class Meta:
        verbose_name = _("Company contact information")
        verbose_name_plural = verbose_name

    def __str__(self):
        return str(_("Client information"))


post_delete.connect(
    auto_purge_files_on_delete(["logo", "logo_invert", "favicon"]),
    dispatch_uid="firm_info_firm_contact_medias_on_delete",
    sender=FirmContact,
    weak=False,
)

pre_save.connect(
    auto_purge_files_on_change(["logo", "logo_invert", "favicon"]),
    dispatch_uid="firm_info_firm_contact_medias_on_change",
    sender=FirmContact,
    weak=False,
)


class Link(models.Model):
    """
    Represents a social network link.

    Attributes:
        SOCIAL_NETWORK_NAMES (tuple): Choices for the name field representing
            various social network names.
        name (CharField): The name of the social network.
        url (URLField): The URL of the social network link.
        client_contact (ForeignKey): The foreign key to the FirmContact model
            representing the client contact.
    """
    SOCIAL_NETWORK_NAMES = (
        ("linkedin", "linkedin"),
        ("facebook", "facebook"),
        ("twitter", "twitter"),
        ("youtube", "youtube"),
        ("instagram", "instagram"),
        ("glassdoor", "glassdoor"),
        ("bitbucket", "bitbucket"),
        ("github", "github"),
        ("gitlab", "gitlab"),
        ("tiktok", "tiktok"),
        ("twitch", "twitch"),
        ("discord", "discord"),
        ("vk", "vk"),
        ("slack", "slack"),
        ("whatsapp", "whatsapp"),
        ("weechat", "weechat"),
    )

    name = models.CharField(max_length=255, choices=SOCIAL_NETWORK_NAMES)
    url = models.URLField(max_length=255)
    client_contact = models.ForeignKey(FirmContact, on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.name} - {self.url}"

    class Meta:
        verbose_name = _("Social network link")
        verbose_name_plural = _("Social networks links")


class SocialSharing(models.Model):
    """
    Represents social media sharing information.

    Attributes:
        og_image (SmartMediaField): The OG image for social media sharing.
        og_description (TextField): The OG description for social media sharing.
        og_twitter_site (CharField): The OG Twitter site for social media sharing.
    """

    og_image = SmartMediaField(
        _("OG Image"),
        null=True,
        blank=True,
        upload_to="firm/social_sharing/%y/%m",
    )
    og_description = models.TextField(
        max_length=180,
        null=False,
        blank=True,
        default="",
        verbose_name=_("OG Description"),
    )
    og_twitter_site = models.CharField(
        max_length=100,
        null=False,
        blank=True,
        default="",
        verbose_name=_("OG Twitter Site"),
    )

    class Meta:
        verbose_name = _("Social media share")
        verbose_name_plural = _("Social media shares")


post_delete.connect(
    auto_purge_files_on_delete(["og_image"]),
    dispatch_uid="socialsharing_medias_on_delete",
    sender=SocialSharing,
    weak=False,
)

pre_save.connect(
    auto_purge_files_on_change(["og_image"]),
    dispatch_uid="socialsharing_medias_on_change",
    sender=SocialSharing,
    weak=False,
)


class Tracking(models.Model):
    """
    Represents tracking information.

    Attributes:
        tag_analytic (CharField): The tag analytic for tracking.
    """

    tag_analytic = models.CharField(
        max_length=100,
        null=False,
        blank=True,
        default="",
        verbose_name=_("Tag Analytic"),
    )

    class Meta:
        verbose_name = _("Tracking")
        verbose_name_plural = _("Tracks")


class AppsBanner(models.Model):
    """
    Represents an app banner in the Django firm_info models.

    Args:
        models.Model: The base model class provided by Django.

    Attributes:
        APPS_CHOICES (list): A list of tuples representing the available choices for
            the application type.
        application_type (models.CharField): The type of the application.
        image (SmartMediaField): The image associated with the app banner.
        title (models.CharField): The title of the app banner.
        description (HTMLField): The description of the app banner.
    """

    APPS_CHOICES = [
        ("application_sent", _("Application sent")),
        ("free_apply", _("Free apply")),
        ("job_offers", _("Job offers")),
        ("news", _("News")),
        ("trombinoscope", _("Trombinoscope")),
    ]

    application_type = models.CharField(
        max_length=16, choices=APPS_CHOICES, unique=True
    )
    image = SmartMediaField(
        _("Image"),
        null=True,
        blank=True,
        upload_to="firm/apps_banner/%y/%m",
    )
    title = models.CharField(_("Title"), max_length=150, blank=True, null=True)
    description = HTMLField(verbose_name=_("Description"), blank=True, null=True)

    def __str__(self):
        return self.get_application_type_display()

    class Meta:
        verbose_name = _("App banner")
        verbose_name_plural = _("App banners")


post_delete.connect(
    auto_purge_files_on_delete(["image"]),
    dispatch_uid="apps_banner_medias_on_delete",
    sender=AppsBanner,
    weak=False,
)

pre_save.connect(
    auto_purge_files_on_change(["image"]),
    dispatch_uid="apps_banner_medias_on_change",
    sender=AppsBanner,
    weak=False,
)



"""
Specific application exceptions.
"""


class MyAppBaseException(Exception):
    """
    Exception base.

    You should never use it directly except for test purpose. Instead make or
    use a dedicated exception related to the error context.
    """
    pass


class SerializeFirmError(MyAppBaseException):
    """
    Exceptions related to FirmContact serialization errors
    during template tag generation.
    """
    pass



# Generated by Django 4.0.10 on 2023-06-09 15:17

from django.db import migrations
import smart_media.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0010_appsbanner'),
    ]

    operations = [
        migrations.AlterField(
            model_name='firmcontact',
            name='favicon',
            field=smart_media.modelfields.SmartMediaField(blank=True, null=True, upload_to='firm_info/firm_contact/favicon/%y/%m', verbose_name='Favicon'),
        ),
        migrations.AlterField(
            model_name='firmcontact',
            name='logo',
            field=smart_media.modelfields.SmartMediaField(blank=True, null=True, upload_to='firm_info/firm_contact/logo/%y/%m', verbose_name='Logo'),
        ),
        migrations.AlterField(
            model_name='firmcontact',
            name='logo_invert',
            field=smart_media.modelfields.SmartMediaField(blank=True, null=True, upload_to='firm_info/firm_contact/logo_invert/%y/%m', verbose_name='Logo invert'),
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _


class SingletonManager(models.Manager):
    """
    A manager to ensure that only one instance of the model exists.

    This manager overrides the `create` method to enforce a singleton pattern
    on the associated model. If an instance of the model already exists,
    attempting to create another instance will raise a `ValueError`.

    Methods:
        create(**kwargs): Creates a new instance of the model if none exists.
            Raises `ValueError` if an instance already exists.
    """
    def create(self, **kwargs):
        if self.model.objects.exists():
            error_message = _("Model {model_name} has already one instance")
            raise ValueError(
                error_message.format(model_name=self.model._meta.verbose_name)
            )
        return super().create(**kwargs)



from django.contrib import admin
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from smart_media.admin import SmartModelAdmin

from .models import AppsBanner, FirmContact, Link, SocialSharing, Tracking


@admin.register(Link)
class LinkAdmin(admin.ModelAdmin):
    pass


class LinkInline(admin.TabularInline):
    model = Link
    extra = 1


@admin.register(FirmContact)
class ClientContactAdmin(admin.ModelAdmin):
    inlines = [LinkInline]
    formfield_overrides = SmartModelAdmin.formfield_overrides

    def has_add_permission(self, request):
        existing_count = FirmContact.objects.count()
        if existing_count == 0:
            return super().has_add_permission(request)
        else:
            return False

    def clean(self):
        existing_count = FirmContact.objects.count()
        if existing_count > 1:
            # raise validation error if there is more than one firm contact
            raise ValidationError(_("Only one FirmContact instance allowed."))


@admin.register(SocialSharing)
class SocialSharingAdmin(admin.ModelAdmin):
    pass


@admin.register(Tracking)
class TrackingAdmin(admin.ModelAdmin):
    pass


@admin.register(AppsBanner)
class AppsBannerAdmin(admin.ModelAdmin):
    formfield_overrides = SmartModelAdmin.formfield_overrides



# Generated by Django 4.0.10 on 2023-05-29 13:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('firm_info', '0005_firmcontact_favicon'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='firmcontact',
            options={'verbose_name': 'Company contact information', 'verbose_name_plural': 'Company contact information'},
        ),
    ]
